uniform float r_oren;
uniform float r_ward;

uniform bool lambert;
uniform bool oren_nayar;
uniform bool ward;

varying vec3 normal;
varying vec3 luz;
varying vec3 observador;
varying vec3 h;

vec4 Lambert(vec3 N, vec3 L){
	
	float NdotL = dot(N, L);
    vec4 Id = gl_LightSource[0].diffuse;
	vec4 rd = gl_FrontMaterial.diffuse;

	return rd*Id*max(0,NdotL);
}

vec4 Oren_Nayar(vec3 N, vec3 L, vec3 V){
	
	float a = max(acos(dot(V,N)), acos(dot(L,N)));
	float b = min(acos(dot(V,N)), acos(dot(L,N)));
	float gama = dot(normalize(V-N*(dot(V,N))), normalize(L-N*(dot(L,N))));
	float C1 = 1 - 0.5*(pow(r_oren,2)/(pow(r_oren,2)+0.33));
	float C2;
	
	if(gama >= 0.0)
		C2 = 0.45*(pow(r_oren,2)/(pow(r_oren,2)+0.09))*sin(a);
	else
		C2 = 0.45*(pow(r_oren,2)/(pow(r_oren,2)+0.09))*(sin(a) - pow((2*b)/3.1415,3));

	float C3 = (1/8)*(pow(r_oren,2)/(pow(r_oren,2)+0.09))*pow(((4*a*b)/pow(3.1415,2)),2);	
	float x = gama*C2*tan(b);
	float e = (1 - abs(gama))*C3*tan((a+b)/2);
	vec4 Id = gl_LightSource[0].diffuse;
	vec4 rd = gl_FrontMaterial.diffuse;
	return rd*Id*dot(N,L)*max(0,(C1 + x + e));
}

vec4 Ward(vec3 N, vec3 L, vec3 V, vec3 H){
	
	float NdotL = dot(N, L);
	float NdotV = dot(N, V);
	float LdotH = dot(L, H);
	float maxNdotL = max(0,NdotL);
	float angulo = acos(LdotH);
	float fator3 = exp(-pow((tan(angulo)/r_ward),2))/
                      (4*3.1415*pow(r_ward,2)*sqrt(NdotL*NdotV));
	vec4 rd = gl_FrontMaterial.specular;

	return rd*maxNdotL*fator3;
}

void main() {
	vec3 N = normalize(normal);
	vec3 L = normalize(luz);
	vec3 V = normalize(observador);
	vec3 H = normalize(h);

	vec4 ambiente = (gl_LightSource[0].ambient * gl_FrontMaterial.ambient)+
				    (vec4(0.1,0.1,0.1,1)*gl_FrontMaterial.ambient);
		
	vec4 difusa_especular;

	if(lambert)
		difusa_especular = Lambert(N,L);
	if(oren_nayar)
		difusa_especular = Oren_Nayar(N,L,V);
	if(ward)
		difusa_especular = difusa_especular + Ward(N,L,V,H);
		
		gl_FragColor = ambiente + vec4(max(difusa_especular.x,0),
									   max(difusa_especular.y,0),
									   max(difusa_especular.z,0),1);
	
}


